@@ -1,7 +1,7 @@
 use strict; use warnings;
 package Inline;
 
-our $VERSION = '0.85';
+our $VERSION = '0.87';
 
 use Inline::denter;
 use Config;
@@ -10,10 +10,16 @@ use Cwd qw(abs_path cwd);
 use Encode;
 use File::Spec;
 use File::Spec::Unix;
-use Fcntl qw(LOCK_EX LOCK_UN);
+use Fcntl qw(LOCK_EX LOCK_SH O_WRONLY O_CREAT);
 use version;
 use utf8;
 
+use constant CAN_FLOCK => !(
+    $^O eq 'VMS' ||
+    $^O eq 'riscos' ||
+    $^O eq 'VOS'
+);
+
 my %CONFIG = ();
 my @DATA_OBJS = ();
 my $INIT = 0;
@@ -559,14 +565,14 @@ END
 sub satisfy_makefile_dep {
     my $o = shift;
 
-       my $inline = $o->{API}{modinlname};
-       open INLINE, "> $inline"
-         or croak M24_open_for_output_failed($inline);
-       print INLINE "*** AUTOGENERATED by Inline.pm ***\n\n";
-       print INLINE "This file satisfies the make dependency for ";
-       print INLINE "$o->{API}{module}\n";
-       close INLINE;
-       return;
+    my $inline = $o->{API}{modinlname};
+    open my $fh, ">", $inline
+     or croak M24_open_for_output_failed($inline);
+    print $fh "*** AUTOGENERATED by Inline.pm ***\n\n";
+    print $fh "This file satisfies the make dependency for ";
+    print $fh "$o->{API}{module}\n";
+    close $fh;
+    return;
 }
 
 #==============================================================================
@@ -668,14 +674,14 @@ sub receive_code {
     }
     elsif ($code =~ m|[/\\:]| and
            $code =~ m|^[/\\:\w.\-\ \$\[\]<>]+$|) {
-        if (-f $code) {
-            local ($/, *CODE);
-            open CODE, "< $code" or croak M06_code_file_failed_open($code);
-            $o->{API}{code} = <CODE>;
-        }
-        else {
-            croak M07_code_file_does_not_exist($code);
-        }
+        open my $fh, "<", $code
+            or croak(
+                $!{ENOENT}
+                    ? M07_code_file_does_not_exist($code)
+                    : M06_code_file_failed_open($code)
+            );
+        local $/;
+        $o->{API}{code} = readline $fh;
     }
     else {
         $o->{API}{code} = $code;
@@ -724,10 +730,8 @@ sub check_config_file {
         my $cwd = Cwd::cwd();
         $DIRECTORY =
           $o->{INLINE}{DIRECTORY} = File::Spec->catdir($cwd, $did);
-        if (not -d $DIRECTORY) {
-            _mkdir($DIRECTORY, 0777)
-              or croak M16_DIRECTORY_mkdir_failed($DIRECTORY);
-        }
+        _mkdir($DIRECTORY, 0777)
+          or croak M16_DIRECTORY_mkdir_failed($DIRECTORY);
     }
     else {
         $DIRECTORY = $o->{INLINE}{DIRECTORY} =
@@ -744,14 +748,13 @@ sub check_config_file {
 
        my $load_cfg = sub {
            $o->create_config_file($DIRECTORY)
-             if not -e File::Spec->catfile($DIRECTORY, $configuration_file);
+             if not -s File::Spec->catfile($DIRECTORY, $configuration_file);
 
-           open CONFIG, "< ".File::Spec->catfile($DIRECTORY,$configuration_file)
+           open my $fh, "<", File::Spec->catfile($DIRECTORY,$configuration_file)
              or croak M17_config_open_failed($DIRECTORY);
-           flock(CONFIG, LOCK_EX) if $^O !~ /^VMS|riscos|VOS$/;
-           my $config = join '', <CONFIG>;
-           flock(CONFIG, LOCK_UN) if $^O !~ /^VMS|riscos|VOS$/;
-           close CONFIG;
+           flock($fh, LOCK_SH) if CAN_FLOCK;
+           my $config = do { local $/; readline $fh };           
+           close $fh;
 
            unless($config =~ /^version :/) {
              warn "\$load_cfg sub: \$config: *${config}*\n";
@@ -805,7 +808,7 @@ sub check_config_file {
 sub derive_minus_I {
     my $o = shift;
     require Cwd;
-    my %libexclude = map { $_=>1 }
+    my @libexclude = (
       # perl has these already
       (grep length, map $Config{$_},
         qw(archlibexp privlibexp sitearchexp sitelibexp vendorarchexp vendorlibexp)),
@@ -813,7 +816,14 @@ sub derive_minus_I {
         map { my $l = $_; ($l, map File::Spec->catdir($l, $Config{$_}), qw(version archname)) }
         split $Config{path_sep}, $ENV{PERL5LIB}
       ) : ()),
-      ;
+    );
+    if ($^O eq 'MSWin32') {
+      # Strawberry Perl Unix-ises its @INC, so we need to add Unix-y versions
+      push @libexclude,
+        map { my $d = $_; $d =~ s#\\#/#g; $d }
+        @libexclude;
+    }
+    my %libexclude = map { $_=>1 } @libexclude;
     my @libinclude = grep !$libexclude{$_}, grep { $_ ne '.' } @INC;
     # grep is because on Windows, Cwd::abs_path blows up on non-exist dir
     @libinclude = map Cwd::abs_path($_), grep -e, @libinclude;
@@ -848,9 +858,9 @@ sub create_config_file {
         %languages, %types, %modules, %suffixes);
     for my $lib (@INC) {
         next unless -d File::Spec->catdir($lib,"Inline");
-        opendir LIB, File::Spec->catdir($lib,"Inline")
+        opendir my $dh, File::Spec->catdir($lib,"Inline")
           or warn(M21_opendir_failed(File::Spec->catdir($lib,"Inline"))), next;
-        while ($mod = readdir(LIB)) {
+        while ($mod = readdir($dh)) {
             next unless $mod =~ /\.pm$/;
             $mod =~ s/\.pm$//;
             next if ($checked{$mod}++);
@@ -877,21 +887,22 @@ sub create_config_file {
             $modules{$language} = "Inline::$mod";
             $suffixes{$language} = $register->{suffix};
         }
-        closedir LIB;
+        closedir $dh;
     }
 
     my $file = File::Spec->catfile($ARGV[0], $configuration_file);
-    open CONFIG, "> $file" or croak M24_open_for_output_failed($file);
-    flock(CONFIG, LOCK_EX) if $^O !~ /^VMS|riscos|VOS$/;
-    print CONFIG Inline::denter->new()
+    sysopen my $fh, $file, O_WRONLY | O_CREAT, 0666
+        or croak M24_open_for_output_failed($file);
+    flock($fh, LOCK_EX) if CAN_FLOCK;
+    truncate($fh, 0);
+    print $fh Inline::denter->new()
       ->indent(*version => $Inline::VERSION,
                *languages => \%languages,
                *types => \%types,
                *modules => \%modules,
                *suffixes => \%suffixes,
               );
-    flock(CONFIG, LOCK_UN) if $^O !~ /^VMS|riscos|VOS$/;
-    close CONFIG;
+    close $fh;
     exit 0;
 }
 
@@ -950,9 +961,10 @@ sub check_module {
                               "$o->{API}{modfname}.$o->{INLINE}{ILSM_suffix}");
         last unless -f $inl;
         my %inl;
-        {   local ($/, *INL);
-            open INL, $inl or croak M31_inline_open_failed($inl);
-            %inl = Inline::denter->new()->undent(<INL>);
+        {
+            open my $fh, "<", $inl or croak M31_inline_open_failed($inl);
+            local $/;
+            %inl = Inline::denter->new()->undent(readline $fh);
         }
         next unless ($o->{INLINE}{md5} eq $inl{md5});
         next unless ($inl{inline_version} ge '0.40');
@@ -1029,10 +1041,10 @@ sub write_inl_file {
     my $inl =
       File::Spec->catfile($o->{API}{install_lib},"auto",$o->{API}{modpname},
                           "$o->{API}{modfname}.inl");
-    open INL, "> $inl"
+    open my $fh, ">", $inl
       or croak "Can't create Inline validation file $inl: $!";
     my $apiversion = $Config{apiversion} || $Config{xs_apiversion};
-    print INL Inline::denter->new()
+    print $fh Inline::denter->new()
       ->indent(*md5, $o->{INLINE}{md5},
                *name, $o->{API}{module},
                *version, $o->{CONFIG}{VERSION},
@@ -1051,7 +1063,7 @@ sub write_inl_file {
                           (apiversion => $apiversion),
                         },
               );
-    close INL;
+    close $fh;
 }
 
 #==============================================================================
@@ -1121,22 +1133,20 @@ sub obj_untaint {
 # Clean the build directory from previous builds
 #==============================================================================
 sub clean_build {
-    use strict;
-    my ($prefix, $dir);
     my $o = shift;
 
-    $prefix = $o->{INLINE}{DIRECTORY};
-    opendir(BUILD, $prefix)
+    my $prefix = $o->{INLINE}{DIRECTORY};
+    opendir(my $dh, $prefix)
       or croak "Can't open build directory: $prefix for cleanup $!\n";
 
-    while ($dir = readdir(BUILD)) {
+    while (my $dir = readdir($dh)) {
         my $maybedir = File::Spec->catdir($prefix,$dir);
         if (($maybedir and -d $maybedir) and ($dir =~ /\w{36,}/)) {
             $o->rmpath($prefix,$dir);
         }
     }
 
-    close BUILD;
+    close $dh;
 }
 
 #==============================================================================
@@ -1161,7 +1171,6 @@ sub filter {
 # User wants to report a bug
 #==============================================================================
 sub reportbug {
-    use strict;
     my $o = shift;
     return if $o->{INLINE}{reportbug_handled}++;
     print STDERR <<END;
@@ -1201,18 +1210,18 @@ END
     }
 
     $o->mkpath($o->{API}{build_dir});
-    open REPORTBUG, "> ".File::Spec->catfile($o->{API}{build_dir},"REPORTBUG")
+    open my $fh, ">", File::Spec->catfile($o->{API}{build_dir},"REPORTBUG")
       or croak M24_open_for_output_failed
                (File::Spec->catfile($o->{API}{build_dir},"REPORTBUG"));
     %Inline::REPORTBUG_Inline_Object = ();
     %Inline::REPORTBUG_Perl_Config = ();
     %Inline::REPORTBUG_Module_Versions = ();
-    print REPORTBUG Inline::denter->new()
+    print $fh Inline::denter->new()
       ->indent(*REPORTBUG_Inline_Object, $o,
                *REPORTBUG_Perl_Config, \%Config::Config,
                *REPORTBUG_Module_Versions, \%versions,
               );
-    close REPORTBUG;
+    close $fh;
 }
 
 #==============================================================================
@@ -1288,7 +1297,7 @@ sub mkpath {
     foreach (@parts){
         push(@done,$_);
         my $path = File::Spec->catpath($volume,File::Spec->catdir(@done),"");
-        -d $path || _mkdir($path, 0777);
+        _mkdir($path, 0777);
     }
     croak M53_mkdir_failed($mkpath)
       unless -d $mkpath;
@@ -1320,9 +1329,9 @@ sub _rmtree {
     foreach $root (@{$roots}) {
         if ( -d $root ) {
             my(@names,@paths);
-            if (opendir MYDIR, $root) {
-                @names = readdir MYDIR;
-                closedir MYDIR;
+            if (opendir my $dh, $root) {
+                @names = readdir $dh;
+                closedir $dh;
             }
             else {
                 croak M21_opendir_failed($root);
@@ -1417,7 +1426,7 @@ sub _mkdir {
     my $mode = shift || 0777;
     ($dir) = ($dir =~ /(.*)/) if UNTAINT;
     $dir =~ s|[/\\:]$||;
-    return mkdir($dir, $mode);
+    return mkdir($dir, $mode) || $!{EEXIST};
 }
 
 #==============================================================================
