@@ -3,10 +3,10 @@ package TestMLBridge;
 use base 'TestML::Bridge';
 
 use Inline;
-use File::Spec::Functions qw(abs2rel);
+use File::Spec::Functions qw(abs2rel catdir);
 
 sub derive_minus_i {
-    my ($self, $perl) = @_;
+    my ($self, $perl, $paths_expected) = @_;
 
     mkdir 'doc';
     mkdir 'eg';
@@ -14,11 +14,12 @@ sub derive_minus_i {
     local @INC = @INC;
     eval $perl;
 
+    my %paths_got = map { abs2rel($_) => 1 } Inline->derive_minus_I;
+
     join '',
         map "$_\n",
-        grep { $_ ne 't' and $_ ne 'inc/lib' }
-        map abs2rel($_),
-        Inline->derive_minus_I;
+        grep $paths_got{$_},
+        split /\n/, $paths_expected;
 }
 
 sub eval_catch {
